/* Fixed point arithmetic expression evaluator (c) by Eric Auer 4/2005 */
/* Intended for inclusion in other programs, for people who hate using */
/* their floating point unit. All others, please use the cool CALC387  */
/* DOS calculator (only 1024 bytes file size!) by Craig Hessel...      */
/* This program shows bad implementations of things meant to be done   */
/* in Assembly language. Still it is kind of useful to have this in C. */

/* This program is provided "AS IS" and WITHOUT ANY EXPRESS OR IMPLIED */
/* WARRANTIES, INCLUDING, without limitation, the implied warranties   */
/* of MERCHANTIBILITY and FITNESS FOR A PARTICULAR PURPOSE.            */

/* License: This program is released under the Artistic License. See   */
/* the internet or your nearest Perl installation for details :-P.     */
/* License summary: You can copy and modify the sources, but must keep */
/* the disclaimer and copyright info in. If you create binaries from   */
/* modified sources, and publish those binaries, you must also publish */
/* the modified sources. Mark modified versions, and only publish them */
/* bundled with information on how to get the original version...      */

/* If you want me to re-release a version of this program under other  */
/* licenses, e.g. BSD, MIT ("just keep copyright info") or GPL / LPGL, */
/* or maybe even Public Domain, tell me why. Mail: eric?coli.uni-sb.de */


#include <stdio.h>		/* printf  */
#include <stdlib.h>		/* strtoul */
#include <string.h>		/* strcat  */
#include <ctype.h>		/* isdigit */

#ifdef __TURBOC__		/* hint: link with prf.c mini printf to get */
	/* UPXed 5kB small binary with Turbo C: tcc -w -f- calc.c prf.c ... */
typedef signed long int sint32;
typedef unsigned long int uint32;
#else	/* with gcc / djgpp, compile as: gcc -s -O2 -Wall calc.c -o calc */
typedef __signed__ int sint32;
typedef unsigned int uint32;
#endif

#ifndef EM_VERBOSE		/* if not already defined in makefile... */
#define EM_VERBOSE 0		/* set to 1 to get verbose progress info */
#endif


#define EM_OVERFLOW -1		/* mul / div overflow */
#define EM_DIVBYZERO -2		/* division by 0 */
#define EM_ILLEGALOP -3		/* illegal operator */
#define EM_PARENFOUND 1		/* closing parenthesis found */
#define EM_PARENMISS 2		/* parenthesis expected */
#define EM_OPERMISS  3		/* operator expected */
#define EM_NUMMISS   4		/* number expected */


				/* buffer for fixpoint - to - string */
static char fpstr[22];		/* does not really HAVE to be global */


/* fixed point to string, provide 1 + 10 + 1 + 9 + 1 = 22 byte buffer: */
void fixpoint_string (char * result, sint32 full, uint32 fract);

/* string to fixed point value, returns post-string pointer or NULL: */
char * parse_value (char * formula, sint32 * full, uint32 * fract);

int add_carry (uint32 a, uint32 b);	/* return 1 if a + b overflows */

int sub_carry (uint32 a, uint32 b);	/* return 1 if a - b overflows */

uint32 mul_big (uint32 a, uint32 b, uint32 * carry); /* 32*32 to 64 bit */

/* evaluate some formula, return results as fixed point values */
/* Does NOT handle operator precedence yet! */
int evalmath (char * formula, sint32 * full, uint32 * fract);

/* evaluate a single fixed point binary operator application */
int binary_operator (char oper, sint32 * full, uint32 * fract, sint32 yfull, uint32 yfract);



/* fixed point to string, provide 1 + 10 + 1 + 9 + 1 = 22 byte buffer: */
void fixpoint_string (char * result, sint32 full, uint32 fract)
{
  uint32 digit, val, part, delta;
  char * first;

  if (result == NULL) return;

  if (full & (1UL << 31)) {	/* negative sign? */
    *result = '-';
    result++;
    full = ~full;		/* toggle e.g. 0.5 versus -0.5 */
    fract = ~fract; 		/* flip sign! */
  }

  /* 0x0fffffffc and above should actually round to 1.0 as we only */
  /* have 9 digits after the comma available. Just doing that now: */
  if (fract >= 0x0fffffffcUL) {
    fract = 0;
    full++;
  }
  if (fract < 4)
    fract = 0;

#ifdef __TURBOC__
  sprintf (result, "%ld%c", full,	/* at most "-2147483647." here */
#else					/* max positive: 2147483647 */
  sprintf (result, "%d%c", full,
#endif
    (fract) ? '.' : 0);			/* decide if there are decimals */

  if (!fract)
    return;				/* skip all decimals-handling... */

  result += strlen (result);

  first = result;
  val = (1UL << 31) / 5;	/* value of "2^32 / 10" */
  part = 6;			/* "0.6" */

  do {
    digit = fract / val;	/* does not include part yet! */

    do {
      if (digit > 9) digit--;		/* can happen because of rounding */
      delta = (digit * val) + (digit * part / 10UL);	/* with part */
      if ( ((part * digit) % 10) >= 5 ) delta++;	/* round up! */
					/* "(part+0.5) * digit / 10" */
      if (delta > fract) digit--;	/* can happen because of rounding */
    } while (delta > fract);

    if (digit > 9) {			/* this CAN happen :-( */
      result--;				/* go back 1 digit */
      /* going back 1 digit is always possible in the first correction step */
      while (*result == '9') {
        *result = 0;			/* shorten (always in range) */
        result--;			/* go back */
        if (result < first) {		/* should never happen */
          strcat(first, "999999...");	/* should round to 1.0 here */
          return;
        }
      } /* while walking along 999s */
      (*result)++;			/* finally, increment a digit */
      result++;
      *result = 0;			/* terminate string */
      return;
    }					/* rounding error ends transform */

    if ((digit < 9) && (val > 4)) {	/* try early cut-off */
      uint32 temp;

      temp = ((digit+1) * val) + ((digit+1) * part / 10UL);
      if ((temp - fract) < 4) {		/* early cut-off of 999... tails */
        digit++;
        delta = fract;
      }
    }					/* cut-off trick */

    *result = (char)('0' + digit);
    result++;

    fract -= delta;

    part = val % 10UL;		/* for more precision */
    val = val / 10UL;		/* next digit please */
  } while ((val != 0) && (fract >= 4));
  /* (fract < 4) is less than ?.000 000 000 5 error already */

  *result = 0;			/* terminate string */
} /* fixpoint_string */



/* returns pointer to after value, or NULL on error... */
char * parse_value (char * formula, sint32 * full, uint32 * fract)
{
  int okay;
  int sign;				/* separate handling needed! */
  char * ptr;

  if ((full == NULL) || (fract == NULL) || (formula == NULL))
    return NULL;

  *full = *fract = okay = 0;
  sign = 1;

  if (isdigit(*formula) || (*formula == '-') || (*formula == '+')) {
    if (*formula == '-')
      sign = -1;
    if ((formula[0] != '0') || (formula[1] != 'x')) {
      *full = strtol (formula, &ptr, 10);
    } else {				/* hex mode */
      formula++;
      formula++;
      *full = strtol (formula, &ptr, 16);
    }
    if (sign < 0) (*full)--;		/* adjust sign, make "-0.5" neg. */
    /* we use "-1" representation "~1" here, which is "-2" in binary...! */
    okay++;
    formula = ptr;			/* first char after value */
  }					/* integer part */

  if (*formula == '.') {		/* fractional part? */
    formula++;				/* skip the dot */
    if ( (formula[0] != '0') || (formula[1] != 'x')) {	/* nonhex? */
      uint32 digit;
      uint32 val, part;
      *fract = 0;
      val = (1UL << 31) / 5;		/* value of "2^32 / 10" */
      part = 6;				/* "0.6" */
      while ( isdigit(*formula) && ( (val != 0) || (part != 0) ) ) {
        digit = *formula - '0';		/* expand to 32 bit */
        *fract += (val * digit) + (part * digit / 10UL);
        if ( ((part * digit) % 10) >= 5 ) (*fract)++;	/* round up! */
        /* "(part+0.5) * digit / 10" */
        formula++;
        part = val % 10UL;		/* for more precision */
        val /= 10UL;
      }
      while (isdigit(*formula))
        formula++;			/* ignore over-exactness */
    } else {
      formula++;
      formula++;
      *fract = strtoul (formula, &ptr, 16);	/* hex mode */
      formula = ptr;			/* first char after value */
    }
    okay++;
  }					/* fractional part */
  if (sign < 0) *fract = ~(*fract);	/* flip sign! */

  while (*formula == ' ') formula++;	/* skip in-between spaces */

  return (okay) ? formula : NULL;

} /* parse_value */



/* returns 1 if 32 bit + 32 bit addition overflows */
int add_carry (uint32 a, uint32 b)
{
  uint32 c;

  c = (a>>1) + (b>>1);
  if (c & (1UL << 31))
    return 1;				/* a clear overflow */
  if ( (a & 1) && (b & 1) && (c == ((1UL << 31) - 1)) )
    return 1;				/* less obvious */
  return 0;				/* else no overflow */
} /* add_carry */



/* returns 1 if 32 bit - 32 bit subtraction overflows */
int sub_carry (uint32 a, uint32 b)
{
  uint32 c;

  c = (a>>1) - (b>>1);
  if (c & (1UL << 31))
    return 1;				/* a clear overflow */
  if ( !(c) && !(a & 1) && (b & 1) )
    return 1;				/* less obvious */
  return 0;				/* else no overflow */
} /* sub_carry */



/* returns 32 + 32 bits of 64 bit result of 32 bit * 32 bit multiply */
/* carry is the high 32 bits, return value is the low 32 bits */
uint32 mul_big (uint32 a, uint32 b, uint32 * carry)
{
  int i;
  uint32 chigh, clow;

  clow = (a & 1) ? b : 0;	/* outside the loop, as >> works modulo 32 */
  chigh = 0;
  for (i = 1; i < 32; i++) {
    if (a & (1UL << i)) {	/* must use "1UL" for Turbo C... */
      chigh += add_carry (clow, (b << i));
      clow += (b << i);
      chigh += (b >> (32-i));	/* (b << (i-32)) */
    }
  }
  *carry = chigh;
  return clow;
} /* mul_big */



/* evaluates formula, returning 0 if okay, also sets full.fract */
/* supported operators: + - * / % and logicals: & | ^ and: ( )  */
/* division only supported by integers at the moment...         */
int evalmath (char * formula, sint32 * full, uint32 * fract)
{
  sint32 fullx, fully;
  uint32 fractx, fracty;
  char * ptr;
  char oper;

  if ( (formula == NULL) || (full == NULL) || (fract == NULL))
    return 42;

  *full = 3; *fract = 608135817UL;	/* 3.14159265(4), for testing */
#if EM_VERBOSE
  printf ("Evalmath %s\n", formula);
#endif

  ptr = parse_value (formula, &fullx, &fractx);
  if (ptr == NULL) {
    while (*formula == ' ') formula++;	/* skip spaces */
    if (*formula == '(') {
      fullx = fully = 0;		/* "0.0 + (...)" */
      oper = '+';
      goto start_with_paren;		/* ** GOTO ** */
    } else {
      return EM_NUMMISS;		/* number expected */
    }
  } else {
    formula = ptr;
  }

  loop_around:				/* ** LABEL ** */

  while (*formula == ' ') formula++;	/* skip leading spaces */

  if (*formula == 0) {			/* done */
    *full = fullx;
    *fract = fractx;
    return 0;
  }

  oper = *formula;
  formula++;

  if (oper == ')') {
#if EM_VERBOSE
    fixpoint_string (fpstr, fullx, fractx);
    printf ("Sub-formula result: %s\n", fpstr);
#endif

    *full = fullx;
    *fract = fractx;
    return EM_PARENFOUND;		/* need not be an error */
  } /* ) found */

  if (strchr ("+-*/%&|^", oper) == NULL) {
    return EM_OPERMISS;			/* operator expected */
  }

  while (*formula == ' ') formula++;	/* skip leading spaces */

  start_with_paren:			/* ** LABEL ** */

  if ((*formula == '(')) {		/* sub-formula instead of number */
    int i;
#if EM_VERBOSE
    fixpoint_string (fpstr, fullx, fractx);
    printf ("Sub-formula: (%s) %c (...)\n", fpstr, oper);
#endif

    formula++;
    i = evalmath (formula, &fully, &fracty);
    if (i != EM_PARENFOUND) {
      if (!i) return EM_PARENMISS;
      return i;				/* abort */
    }

    i = 1;				/* depth count */
    while ((*formula != 0) && (i)) {
      if (*formula == '(') i++;
      if (*formula == ')') i--;
      formula++;
    }

  } else {				/* just a number as 2nd operand */

    ptr = parse_value (formula, &fully, &fracty);
    if (ptr == NULL) return EM_NUMMISS;	/* number expected */
    formula = ptr;

  }

#if EM_VERBOSE
  {
    fixpoint_string (fpstr, fullx, fractx);
#ifdef __TURBOC__
    printf ("Doing %s [%lx:%08lx] %c ", fpstr, fullx, fractx, oper);
    fixpoint_string (fpstr, fully, fracty);
    printf ("%s [%lx:%08lx] = ...\n", fpstr, fully, fracty);
#else
    printf ("Doing %s [%x:%08x] %c ", fpstr, fullx, fractx, oper);
    fixpoint_string (fpstr, fully, fracty);
    printf ("%s [%x:%08x] = ...\n", fpstr, fully, fracty);
#endif
  }
#endif

  switch (binary_operator (oper, &fullx, &fractx, fully, fracty)) {
    case EM_OVERFLOW:  printf ("Overflow!\n"); break;
    case EM_DIVBYZERO: printf ("Division by zero!\n"); break;
    case EM_ILLEGALOP: printf ("Unsupported operator!\n"); break;
    case 0:            break; /* okay */
    default:           printf ("Cannot calculate result!\n");
  } /* switch */

  goto loop_around;			/* ** GOTO ** */

} /* evalmath */



/* calculate the result of a single binary math or logical operation... */
/* return nonzero on error. "full.fract = full.fract oper yfull.yfract" */
/* note that divide and modulo can involve a significant precision loss */
int binary_operator (char oper, sint32 * full, uint32 * fract, sint32 yfull, uint32 yfract)
{
  int shift;
  int sign1, sign2;

  switch (oper) {
    case '&':				/* AND */
      *fract &= yfract;
      *full &= yfull;
      return 0;
      /* break; */

    case '|':				/* OR */
      *fract |= yfract;
      *full |= yfull;
      return 0;
      /* break; */

    case '^':				/* XOR */
      *fract ^= yfract;
      *full ^= yfull;
      return 0;
      /* break; */
  } /* switch - the binary ones */

  sign1 = sign2 = 0;			/* split off signs here */
  if (*full & (1UL << 31)) {
    sign1++;
    (*full) = ~(*full);			/* flip special sign */
    (*fract) = ~(*fract);		/* flip sign here, too! */
  }
  if (yfull & (1UL << 31)) {
    sign2++;
    yfull = ~yfull;			/* flip special sign */
    yfract = ~yfract;			/* flip sign here, too! */
  }

  switch (oper) {
    case '-':				/* SUB */ /* add the complement */
      sign2 = 1 - sign2;
      /* no break - falling through to add here! */

    case '+':				/* ADD */
      if (sign1 == sign2) {		/* stay on one side of 0 */

        (*full)  += yfull;
        if (add_carry (*fract, yfract)) (*full)++;
        (*fract) += yfract;

        if ((*full) & (1UL << 31))	/* unwanted sign change? */
          return EM_OVERFLOW;

        if (sign1) {			/* result negative? */
          (*full) = ~(*full);		/* flip special sign */
          (*fract) = ~(*fract);		/* flip sign here, too! */
        }
        
      } else {				/* different signs: subtract */
        int carry;

        if (sign1) {			/* negative plus positive: swap */
          sint32 fu;
          uint32 fr;

          fu = (*full);
          fr = (*fract);
          (*full) = yfull;
          (*fract) = yfract;
          yfull = fu;
          yfract = fr;
        }				/* swap to get "*" - "y" order */
#if 0
?       yfract++;			/* fixup !? */
#endif
        carry = sub_carry (*fract, yfract);
        /* printf ("Carry: %lx - %lx = %d,\n", *fract, yfract, carry); */
        (*fract) -= yfract;
        if (carry) {
          sign1 = (!(*full)) ? 1 : 0;	/* set if result negative */
          (*full)--;
        }
        (*full) -= yfull;
        /* result sign already happens to be correct at this moment !? */
      }
      break;

    case '*':				/* MUL */ /* precision loss risks?? */
      {
        uint32 resultah, ishuge, resultbl, resultbh, resultcl, resultch, resultdl;
        /* "huge" would be a reserved word for Turbo C. No problem with GNU C. */
	/* decimal example: 1.9 * 1.8 = 1*1 + 1*0.9 + 0.8*1 + 0.9*0.8 */
        resultah = mul_big (*full, yfull, &ishuge);
        if (ishuge) return EM_OVERFLOW;	/* overflow */
          /* "resultal" is 0 */
        resultbl = mul_big (*full, yfract, &resultbh);
        resultcl = mul_big (yfull, *fract, &resultch);
        if (mul_big (*fract, yfract, &resultdl) & (1UL << 31))
          resultdl++;	/* round, for better precision */
          /* throw away the other too insignificant result bits */
          /* "resultdh" is 0 */
        *full = resultah;
        *fract = resultbl + resultcl;	/* "resultal" is 0 ... */
        if (add_carry (resultbl, resultcl)) {
          (*full)++;
          if (!(*full)) return EM_OVERFLOW;
        }
        if (add_carry (*fract, resultdl)) {
          (*full)++;
          if (!(*full)) return EM_OVERFLOW;
        }
        *fract += resultdl;		/* resultbl + resultcl + resultdl */
        if (add_carry (*full, resultbh)) return EM_OVERFLOW;
        *full += resultbh;
        if (add_carry (*full, resultch)) return EM_OVERFLOW;
        *full += resultch;		/* resultah + resultbh + resultch */
        /* ... "resultdh" is 0 */
        if (sign1 ^ sign2) {		/* result negative? */
	  (*full) = ~(*full);		/* store special sign */
	  (*fract) = ~(*fract);		/* flip sign here, too! */
        }
      }
      break;

    case '/':				/* DIV */ /* take care, precision loss! */
    case '%':				/* MOD */ /* take care, precision loss! */
      if ((yfull == 0) && (yfract == 0)) {	/* division by 0 */
        if (oper == '%') {
          *full = *fract = 0;			/* modulo by zero is "okay" */
          return 0;
        }
        return EM_DIVBYZERO;			/* real division by zero */
      }
      shift = 0;
      if (yfract != 0) {		/* force to divide by integer */
        /* double "y" until yfract is 0 */
        /* double "*" at the same time to keep sync */
        while (!(*full & (1UL << 30)) &&
               !(yfull & (1UL << 30)) ) { /* as far as sint32 possible */
          *full <<= 1;
          if (*fract & (1UL << 31)) (*full)++;	/* do 64bit shift */
          *fract <<= 1;
          yfull <<= 1;
          if (yfract & (1UL << 31)) yfull++;	/* do 64bit shift */
          yfract <<= 1;
          shift++;			/* remember for modulo */
          if (!yfract) break;		/* fractional part already 0 */
        }
        if (yfract) {
          /* precision loss is equal to log2 of original *full, or */
          /* less if only the high bits of yfract were used...     */
          /* solution would be to make *full "short" by splitting  */
          /* the divide operation into per-bit-of-*full rounds...  */

          if (yfract & (1UL << 31)) yfull++;	/* round up, more exact! */
#if EM_VERBOSE
          if (yfract & ((1UL << 16)-1))		/* half+ of fract pending? */
            printf ("Big precision loss!\n.");	/* i.e. fract part wrong */
#endif
          yfract = 0;			/* force to 0 */
        }
      }					/* force division to be by integer */
      {					/* main divide and modulo stuff */
        sint32 divi, remi;
        int i, j;

        if (!yfull)			/* happens if "big / 0.small" */
          return EM_OVERFLOW;		/* division overflow */

        divi = *full / yfull;		/* division result */
        remi = *full - (yfull * divi);	/* "odd" part of result */

        if (oper == '%') {		/* modulo, not division */
          /* *fract = *fract */		/* we only divided by an integer */
          *full = remi;
          while (shift) {		/* adjust back */
            *fract >>= 1;
            if (*full & 1) *fract |= (1UL << 31);
            *full >>= 1;
            shift--;
          }
          if (sign1 ^ sign2) {		/* result negative? */
	    (*full) = ~(*full);		/* store special sign */
	    (*fract) = ~(*fract);	/* flip sign here, too! */
          }
          break;
        }

        *full = divi;
        /* fractional part: "remi.*fract / yfull" */
        /* 0.((*fract/yfull) + (remi/yfull) ...   */
        /* remi / yfull is always 0.something ... */
        *fract /= yfull;

        /* e.g. 7fffffff/40000000 = 1 + (3fffffff/40000000) = 1.999... */
        for (i = 31, j = 0; i && remi; i--) {	/* bit-by-bit division */
          uint32 extra;

          j++;				/* must be >= in the line below! */
          extra = ( ((uint32)(remi << j)) >= yfull) ? (1UL << i) : 0;
          /* printf("remi=%x extra=%x i=%d j=%d\n", remi, extra, i, j); */
          /* must be "1UL << i" for Turbo C, "1 << i" does not work...! */
          *full += add_carry (*fract, extra);	/* ever used? */
          *fract += extra;
          if (extra) {			/* hit a "1" bit position */
            remi = ((uint32)(remi << j)) - yfull;
            j = 0;
          }
        } /* division loop */

        /* printf("remi=%x j=%d yfull=%d\n", remi, j, yfull); */
        if ( (remi << j) >= (yfull >> 1) ) {	/* final rounding */
          (*fract)++;
          if (!(*fract)) (*full)++;	/* carry handling */
        }

        if (sign1 ^ sign2) {		/* result negative? */
          (*full) = ~(*full);		/* store special sign */
          (*fract) = ~(*fract);		/* flip sign here, too! */
        }
      }
      break;

    default:
      return EM_ILLEGALOP;		/* invalid operator char */
  } /* switch */

  return 0;				/* everything okay */

} /* binary_operator */



/* a really crappy main to fetch the expression */
int main (int argc, char **argv)
{
  char ex[1000];		/* yes, it can overflow. evil. */
  int i;
  sint32 a;
  uint32 b;

  if (argc < 2) {
    printf ("Usage: calc \"math_expression\" - shows the result.\n");
    printf ("Example: calc \"5.5 + 0.01 * (7.35 / 3) + 42\"\n");
    return 1;
  }

  ex[0] = 0;
  for (i=1; i < argc; i++) {
    strcat (ex, argv[i]);	/* yes, it can overflow. evil. */
    strcat (ex, " ");		/* yes, it can overflow. evil. */
  }

  /* printf ("Expression to be evaluated: \n%s\n", ex); */
  a = 0;
  b = 0;
  i = evalmath (ex, &a, &b);
  if (i) {
    printf ("Expression evaluation error: ");
    switch (i) {
      case EM_OVERFLOW:   printf ("Overflow!\n"); break;
      case EM_DIVBYZERO:  printf ("Division by zero!\n"); break;
      case EM_ILLEGALOP:  printf ("Illegal operator!\n"); break;
      case EM_PARENFOUND: printf ("Too many closing ()...\n"); break;
      case EM_PARENMISS:  printf ("Too many opening ()...\n"); break;
      case EM_OPERMISS:   printf ("Operator expected.\n"); break;
      case EM_NUMMISS:    printf ("Number expected.\n"); break;
      default:            printf ("Internal error %d.\n", i);
    } /* switch */
    fixpoint_string (fpstr, a, b);
#ifdef __TURBOC__
    printf ("Partial result: %s [0x%lx:%08lx]\n", fpstr, a, b);
#else
    printf ("Partial result: %s [0x%x:%08x]\n", fpstr, a, b);
#endif
    return 2;
  }

  fixpoint_string (fpstr, a, b);
#ifdef __TURBOC__
  printf ("Result: %s [0x%lx:%08lx]\n", fpstr, a, b);
#else
  printf ("Result: %s [0x%x:%08x]\n", fpstr, a, b);
#endif

  return 0;  
} /* main */


