; PROGRAM   : CRC32 Calculator
; DATE      : 15-Feb-2004
;
; LICENSE   : public domain, "as is"
; AUTHOR    : Joe Forster/STA
;
; CONTACT   : sta@c64.org
; WEBPAGE   : http://sta.c64.org
;
; ASSEMBLER : NASM (formerly TASM)

%idefine DOS int 21h

BufferSize equ 64512
ProgramSize equ 4096

; named constants
CR equ 13
LF equ 10
TAB equ 9
ARGV equ 81h
STDERR equ 2
AVAILBYTES equ 0006h
POLYNOMIAL equ 0EDB88320h
ARCHIVE equ 1 << 5
SYSTEM equ 1 << 2
HIDDEN equ 1 << 1
READONLY equ 1

; function calls
ALLOCATE equ 48h
CLOSE equ 3Eh
EXIT equ 4Ch
FINDFIRST equ 4Eh
FINDNEXT equ 4Fh
FREE equ 49h
OPENREAD equ 3D00h
READFILE equ 3Fh
REALLOCATE equ 4Ah
SEEKEND equ 4202h
SEEKSET equ 4200h
WRITEFILE equ 40h
WRITESTR$ equ 9

; ------------------------------------------------------------
cpu 8086
section .text
org 100h
; ------------------------------------------------------------

Main:
  mov di,DataStart
  mov cx,DataEnd - DataStart
  xor al,al
  cld
  rep stosb
  mov ax,[AVAILBYTES]
  cmp ax,ProgramSize
  jbe .sized
  mov bx,ProgramSize / 16
  mov ah,REALLOCATE
  DOS
  mov sp,ProgramSize
  xor ax,ax
  push ax
.sized:
  mov bl,1
  mov di,SearchPar
  mov cx,79
  call ParamStr
  or bh,bh
  jne .01
  mov dx,Hello
  mov ah,WRITESTR$
  DOS
  mov dx,Usage
  mov ah,WRITESTR$
  DOS
  xor al,al
  jmp .03
.01:
  mov si,SearchPar
  mov di,ReadPath
  mov bx,ReadName
  mov bp,ReadExt
  call FSplit
  mov si,ReadPath
  mov di,SearchPar
  mov cx,79
  clc
  call StrCopy
  mov si,ReadName
  clc
  call StrCopy
  mov si,ReadExt
  stc
  call StrCopy
  mov dx,SearchPar
  mov cx,ARCHIVE | SYSTEM | HIDDEN | READONLY
  mov ah,FINDFIRST
  DOS
  jnc .06
  mov dx,SearchPar
  clc
  call PrintStr
  mov dx,NotFound
  jmp .02
.06:
  call MakeCRC
  mov si,9Eh
  xor di,di
  mov bx,ReadName
  mov bp,ReadExt
  call FSplit
  mov si,ReadPath
  mov di,SearchPar
  mov cx,79
  clc
  call StrCopy
  mov si,9Eh
  stc
  call StrCopy
  mov dx,SearchPar
  mov ax,OPENREAD
  DOS
  jnc .07
  jmp .04
.07:
  mov si,ax
  mov bx,BufferSize / 16
  mov ah,ALLOCATE
  DOS
  jnc .10
  mov dx,NoMemory
  jmp .02
.10:
  mov [Buffer],ax
  xor ax,ax
  mov word [CRC],ax
  mov word [CRC+2],ax
  mov bx,si
  mov ax,SEEKEND
  xor cx,cx
  xor dx,dx
  DOS
  mov word [ReadLen],ax
  mov word [ReadLen+2],dx
  mov ax,SEEKSET
  xor cx,cx
  xor dx,dx
  DOS
  xor ax,ax
  mov word [Pos],ax
  mov word [Pos+2],ax
.14:
  mov ax,word [ReadLen]
  mov dx,word [ReadLen+2]
  mov bx,word [Pos]
  mov cx,word [Pos+2]
  sub ax,bx
  sbb dx,cx
  jc .15
  or dx,dx
  jne .16
  or ax,ax
  je .15
  cmp ax,BufferSize
  jbe .17
.16:
  mov ax,BufferSize
.17:
  mov cx,ax
  mov ax,word [Pos]
  mov dx,word [Pos+2]
  add ax,cx
  adc dx,byte 0
  mov word [Pos],ax
  mov word [Pos+2],dx
  mov bx,si
  mov ax,[Buffer]
  call BlockRead
  push si
  push cx
  push es
  xor si,si
  mov di,cx
  mov cx,word [CRC]
  mov dx,word [CRC+2]
  not cx
  not dx
  mov es,[Buffer]
.05:
  xor bx,bx
  mov bl,[es:si]
  xor bl,cl
  mov cl,ch
  mov ch,dl
  mov dl,dh
  xor dh,dh
  times 2 shl bx,1
  xor cx,word [CRCTable+bx]
  xor dx,word [CRCTable+bx+2]
  inc si
  dec di
  jne .05
  pop es
  not cx
  not dx
  mov word [CRC],cx
  mov word [CRC+2],dx
  pop cx
  pop si
  jmp .14
.15:
  mov dx,SearchPar
  clc
  call PrintStr
  mov dl,TAB
  mov ah,6
  DOS
  mov ax,word [CRC]
  mov dx,word [CRC+2]
  mov cx,830h
  call HexStr
  stc
  call PrintStr
  push es
  mov es,[Buffer]
  mov ah,FREE
  DOS
  pop es
.09:
  mov bx,si
  mov ah,CLOSE
  DOS
.04:
  mov ah,FINDNEXT
  DOS
  jc .03
  jmp .06
.02:
  mov ah,WRITESTR$
  DOS
  mov al,1
  jmp short .19
.03:
  xor al,al
.19:
  mov ah,EXIT
  DOS


; proc
MakeCRC:
  xor si,si
  mov di,CRCTable
  cld
.03:
  mov ax,si
  xor bx,bx
  mov cx,8
.02:
  shr bx,1
  rcr ax,1
  jnc .01
  xor ax,POLYNOMIAL & 0FFFFh
  xor bx,POLYNOMIAL >> 16
.01:
  loop .02
  stosw
  mov ax,bx
  stosw
  inc si
  cmp si,256
  jbe .03
  ret

; proc
BlockRead:
  push ds
  push dx
  mov ds,ax
  xor dx,dx
  mov ah,READFILE
  DOS
  pop dx
  pop ds
  ret

; proc
HexStr:
  mov bx,16
  jmp short NumStr
DecStr:
  mov bx,10
  xor cx,cx
NumStr:
  push si
  push di
  push cx
  mov si,bx
  mov di,NumStrBuf+10
  mov byte [di],0
  mov cx,di
  mov bx,dx
.01:
  xor dx,dx
  xchg bx,ax
  div si
  xchg bx,ax
  div si
  add dl,'0'
  cmp dl,'9'
  jbe .02
  add dl,'A'-'0'-10
.02:
  dec di
  mov [di],dl
  mov dx,ax
  or dx,bx
  jne .01
  sub cx,di
  pop bx
  or bh,bh
  je .03
  sub bh,cl
  jbe .03
.04:
  dec di
  mov byte [di],bl
  inc cx
  dec bh
  jne .04
.03:
  mov dx,di
  pop di
  pop si
  ret

; proc
FSplit:
  mov dx,si
  call StrLen
  add si,ax
  dec si
  xor cx,cx
  std
.04:
  cmp si,dx
  jb .01
  lodsb
  cmp al,'.'
  jne .02
  or cx,cx
  jne .02
  mov cx,si
  inc cx
.02:
  cmp al,'\'
  je .03
  cmp al,':'
  jne .04
.03:
  inc si
.01:
  inc si
  or cx,cx
  jne .05
  mov cx,dx
  xchg si,dx
  call StrLen
  xchg si,dx
  add cx,ax
.05:
  xchg si,dx
  sub cx,dx
  sub dx,si
  push cx
  mov cx,67
  stc
  call StrPartCopy
  mov di,bx
  pop dx
  mov cx,8
  stc
  call StrPartCopy
  mov di,bp
  mov cx,4
  stc
  call StrCopy
  ret

; proc
StrCopy:
  mov dx,cx

; proc
StrPartCopy:
  cld
  pushf
  or di,di
  jne .04
  popf
  ret
.04:
  or cx,cx
  je .01
.02:
  or dx,dx
  je .01
  lodsb
  or al,al
  je .01
  stosb
  dec dx
  loop .02
.01:
  xor al,al
  popf
  jnc .03
  stosb
.03:
  ret

; proc
StrLen:
  cld
  push si
  xor ah,ah
.02:
  lodsb
  or al,al
  je .01
  inc ah
  or ah,ah
  jne .02
.01:
  pop si
  mov al,ah
  xor ah,ah
  ret

; proc
PrintStr:
  push si
  push bx
  push cx
  pushf
  mov si,dx
  mov bx,1
  call StrLen
  or ax,ax
  je .01
  mov cx,ax
  mov ah,WRITEFILE
  DOS
.01:
  popf
  jnc .02
  mov dx,LineFeed
  mov cx,STDERR
  mov ah,WRITEFILE
  DOS
.02:
  pop cx
  pop bx
  pop si
  ret

; proc
ParamStr:
  mov si,ARGV
  xor bh,bh
.01:
  lodsb
  cmp al,CR
  je .02
  cmp al,' '
  je .01
  cmp al,TAB
  je .01
  inc bh
  cmp bl,bh
  je .03
.04:
  cmp al,CR
  je .02
  cmp al,' '
  je .01
  cmp al,TAB
  je .01
  lodsb
  jmp short .04
.03:
  or di,di
  je .05
.06:
  cmp al,CR
  je .02
  cmp al,' '
  je .02
  cmp al,TAB
  je .02
  call UpCase
  stosb
  lodsb
  loop .06
.02:
  xor al,al
  stosb
.05:
  ret

; proc
UpCase:
  cmp al,'a'
  jb .01
  cmp al,'z'
  ja .01
  sub al,'a'-'A'
.01:
  ret

; ------------------------------------------------------------
section .data align=1
; ------------------------------------------------------------

Hello    db 'CRC32 Calculator by Joe Forster/STA',CR,LF,CR,LF,'$'
Usage    db 'This program calculates the 32-bit CRC checksum of'
         db ' binary files.',CR,LF,CR,LF
         db 'Usage: CRC32 <filename>',CR,LF,'$'
NotFound db ' not found',CR,LF,'$'
NoMemory db 'Out of memory',CR,LF,'$'
LineFeed db CR,LF

; ------------------------------------------------------------
section .bss align=1
; ------------------------------------------------------------

DataStart:

ReadLen   resd 1
Pos       resd 1
CRC       resd 1

Buffer    resw 1
BufferLen resw 1

SearchPar resb 79 + 1
ReadPath  resb 79 + 1
ReadName  resb 8 + 1
ReadExt   resb 4 + 1
NumStrBuf resb 10 + 1

CRCTable  resd 256

DataEnd:

; EOF
