/*
======================================================================

  ZERO.C -- naively writes zeroes to file to clear deleted drive data

======================================================================

    ** LICENSE! **
       = public domain, nenies proprajho, free for any use

    ** AUTHOR!  **
       = rugxulo _AT_ gmail

    ** BUGS!    **
       = only works on FAT12/FAT16 (max. 2 GB) drives

    ** TODO!    **
       1). rewrite!! (but currently using FPC/16-bit "TP" version)

           ... in assembly (see Eric's FREETEST)

           ... or with DJGPP (see /current/v2/DJLSR205.ZIP's statfs.c)

           = we need to use int 21h, 7303h here
             = (if available, else fallback to int 21h 36h)

           = N.B. some tools (even DJGPP 2.05) still only work
                  correctly if DOS kernel version >= 7.x
             = try 2.05's FileUtils: /current/v2gnu/FIL41B.ZIP's df.exe

    ** HISTORY! **
       = 0.1 -- initial version (very rough and simplistic)

       = 0.2 -- not released (email only)
             -- compatible with more compilers

       = 0.3 -- now "VERBOSE" is runtime, not compile time
             -- small size increase, but who cares??

       = 0.4 -- now can select which drive to zero at runtime
             -- minor cleanups (no more gotos, renamed variables)

       = 0.5 -- very minor cleanups

======================================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <dos.h>

#if !defined __WATCOMC__ && !defined __DJGPP__ && \
    !defined __CCDL__ && !defined __ORANGEC__ && \
    !defined LSI_C /* and also MS C ?? */
#error Bad command or filename
#endif

typedef unsigned long bigdisk;

#define DISK(D,WHAT) (bigdisk)D.WHAT##_clusters \
        * (bigdisk)D.sectors_per_cluster \
        * (bigdisk)D.bytes_per_sector

#define DISKMSG "%10lu bytes "

#define CHUNKSIZE 32000
#define CHUNKMSG "%lu" /* "%08X" */

unsigned char buf[CHUNKSIZE] = { 0 };

void show(char* s, bigdisk bd) {
  if (getenv("VERBOSE") != NULL) printf(s,bd);
}

int main(int argc, char** argv) {
  struct diskfree_t mydisk; bigdisk avail,total; unsigned mydrive;
  FILE* fp; int rc=EXIT_FAILURE; char zero[] = "?:\\zero.dat";
  int testonly=0;

  _dos_getdrive(&mydrive); zero[0] = 'A'-1 + mydrive;

  if (argc > 1) {
    testonly = (argv[1][0] == '/' || argv[1][0] == '-') &&
      (strcmp(&argv[1][1],"test") == 0);

    if (!testonly) {
        zero[0] = toupper(argv[1][0]); mydrive = zero[0]-'A'+1;
    }
  }

  puts(zero);

  if (_dos_getdiskfree(mydrive,&mydisk) == 0) {

    avail = DISK(mydisk,avail); total = DISK(mydisk,total);
  
    memset(buf,0,sizeof(buf));
  
    printf(DISKMSG "available\n",avail);
    printf(DISKMSG "total\n",total);

    if (!testonly && ((fp = fopen(zero,"wb")) != NULL)) {
      do {
        show(CHUNKMSG "\n",avail);
        fwrite(buf,sizeof(buf),1,fp);
        avail -= CHUNKSIZE;
      } while (avail >= CHUNKSIZE);
    
      show("Final chunk = " CHUNKMSG "\n",avail);
      if (avail) fwrite(buf,avail,1,fp);
    
      fclose(fp); remove(zero);
    
      rc = EXIT_SUCCESS;
    }

  }

  return rc;
}

/* EOF */
